# 3.2 i18n国际化


在Knife4j `2.0.3`版本提供了i18n的支持,目前支持的语言主要包含2种：中文(zh-CN)、英文(en-US)。Knife4j默认是中文

那么,如何使用呢?

## 3.2.1 文档中选择

通过访问`doc.html`打开文档界面,可以在文档的右上角看到语言的选择,如下图：

![](/knife4j/images/knife4j/i18n.png)

直接选择相应的语言版本即可

## 3.2.2 通过文档地址直接默认i18n

Knife4j也提供了通过地址栏直接将文档显示为指定的语言显示 

访问地址规则：

- 中文：`http://host:port/doc.html#/home/zh-CN`
- 英文：`http://host:port/doc.html#/home/en-US`

另外,如果你是使用了`knife4j`提供的[增强功能](autoEnableKnife4j.md),你也可以这样访问


- 中文：`http://host:port/doc.html#/plus/zh-CN`
- 英文：`http://host:port/doc.html#/plus/en-US`

 
## 3.2.3 服务端yml配置设定


 当然，开发者也可以在后端控制文档语言呈现方式(自2.0.6版本开始)
 
 yml配置如下：
```yml
knife4j:
  enable: true
  setting:
    # Knife4j默认显示中文(zh-CN),如果开发者想直接显示英文(en-US)，在通过该配置进行设置即可
    language: zh-CN
```

在配置文件中进行配置后,还需要在创建Docket对象时，通过调用Knife4j提供的工具对象`OpenApiExtensionResolver`提供一个OpenAPI的扩展属性结构


示例代码如下：

::: details 点击查看代码
```java
@Configuration
@EnableSwagger2WebMvc
public class SwaggerConfiguration {

   private final OpenApiExtensionResolver openApiExtensionResolver;

    @Autowired
    public SwaggerConfiguration(OpenApiExtensionResolver openApiExtensionResolver) {
        this.openApiExtensionResolver = openApiExtensionResolver;
    }

    @Bean(value = "defaultApi2")
    public Docket defaultApi2() {
        String groupName="2.X版本";
        Docket docket=new Docket(DocumentationType.SWAGGER_2)
                .host("https://www.baidu.com")
                .apiInfo(apiInfo())
                .groupName(groupName)
                .select()
                .apis(RequestHandlerSelectors.basePackage("com.swagger.bootstrap.ui.demo.new2"))
                .paths(PathSelectors.any())
                .build()
                .extensions(openApiExtensionResolver.buildExtensions(groupName));
        return docket;
    }

    private ApiInfo apiInfo() {
        return new ApiInfoBuilder()
                //.title("swagger-bootstrap-ui-demo RESTful APIs")
                .description("# swagger-bootstrap-ui-demo RESTful APIs")
                .termsOfServiceUrl("http://www.xx.com/")
                .contact("xx@qq.com")
                .version("1.0")
                .build();
    }
}
```
:::
通过上面示例代码，主要步骤如下：

1、通过`@Autowired`注解引入`Knife4j`向Spring容器注入的Bean对象`OpenApiExtensionResolver`

2、最终在`Dcoket`对象构建后，通过调用`Docket`对象的`extensions`方法进行插件赋值

3、插件赋值需要调用`OpenApiExtensionResolver`提供的`buildExtensions`方法，该方法需要一个逻辑分组名称，就是开发者在`yml`配置文件中配置的`group`名称

::: tip
为什么需要这么做?这样做的目的一方面是充分利用Spring Boot提供的配置方式，方便开发者自定义配置属性，另一方面，通过扩展OpenAPI的规范属性，也更加符合OpenAPI对于扩展属性的要求

OpenAPI规范明确规定,对于扩展属性,开发者应当在响应的某个节点中，增加`x-`开头的属性方式,以扩展自定义的属性配置

自定义文档的扩展属性，开发者可以通过浏览器的Network功能查看OpenAPI的结构，最终Knife4j扩展增加`x-setting`属性，代表增加的扩展自定义文档属性，最终在Ui界面解析呈现，结构如下图：

![](/knife4j/images/documentation/language.png)
:::

 