# 3.25 JSR303

JSR303规范注解在目前的Knife4j界面中做的比较隐晦，主要是考虑到并没有一个好的交互方式进行呈现，开发者可以通过在接口文档的请求参数栏进行查看

在接口文档的请求参数中，针对每个参数的数据类型，如果该参数包含JSR303注解规范，则会以下滑曲线的方式呈现，效果如下图：

![](/knife4j/images/enhance/jsr303.png)

开发者如果要使用JSR303注解，需要在创建Swagger的配置文件中导入springfox的关于JSR303的配置文件，如下：
```java
@Configuration
@EnableSwagger2WebMvc
@Import(BeanValidatorPluginsConfiguration.class)
public class SwaggerConfiguration {
    //more...
}
```
`BeanValidatorPluginsConfiguration`是核心，Springfox项目主要是通过spring-plugin插件体系对JSR303提供支持，而`BeanValidatorPluginsConfiguration`配置文件则是将和JSR303相关的实体Plugin实现注入到Spring的容器中

代码如下：
```java
@Configuration
public class BeanValidatorPluginsConfiguration {

  @Bean
  public ExpandedParameterMinMaxAnnotationPlugin expanderMinMax() {
    return new ExpandedParameterMinMaxAnnotationPlugin();
  }

  @Bean
  public ExpandedParameterNotNullAnnotationPlugin expanderNotNull() {
    return new ExpandedParameterNotNullAnnotationPlugin();
  }

  @Bean
  public ExpandedParameterNotBlankAnnotationPlugin expanderNotBlank() {
    return new ExpandedParameterNotBlankAnnotationPlugin();
  }

  @Bean
  public ExpandedParameterPatternAnnotationPlugin expanderPattern() {
    return new ExpandedParameterPatternAnnotationPlugin();
  }

  @Bean
  public ExpandedParameterSizeAnnotationPlugin expanderSize() {
    return new ExpandedParameterSizeAnnotationPlugin();
  }

  @Bean
  public springfox.bean.validators.plugins.parameter.MinMaxAnnotationPlugin parameterMinMax() {
    return new springfox.bean.validators.plugins.parameter.MinMaxAnnotationPlugin();
  }

  @Bean
  public springfox.bean.validators.plugins.parameter.NotNullAnnotationPlugin parameterNotNull() {
    return new springfox.bean.validators.plugins.parameter.NotNullAnnotationPlugin();
  }

  @Bean
  public springfox.bean.validators.plugins.parameter.NotBlankAnnotationPlugin parameterNotBlank() {
    return new springfox.bean.validators.plugins.parameter.NotBlankAnnotationPlugin();
  }

  @Bean
  public springfox.bean.validators.plugins.parameter.PatternAnnotationPlugin parameterPattern() {
    return new springfox.bean.validators.plugins.parameter.PatternAnnotationPlugin();
  }

  @Bean
  public springfox.bean.validators.plugins.parameter.SizeAnnotationPlugin parameterSize() {
    return new springfox.bean.validators.plugins.parameter.SizeAnnotationPlugin();
  }

  @Bean
  public MinMaxAnnotationPlugin minMaxPlugin() {
    return new MinMaxAnnotationPlugin();
  }

  @Bean
  public DecimalMinMaxAnnotationPlugin decimalMinMaxPlugin() {
    return new DecimalMinMaxAnnotationPlugin();
  }

  @Bean
  public SizeAnnotationPlugin sizePlugin() {
    return new SizeAnnotationPlugin();
  }

  @Bean
  public NotNullAnnotationPlugin notNullPlugin() {
    return new NotNullAnnotationPlugin();
  }

  @Bean
  public NotBlankAnnotationPlugin notBlankPlugin() {
    return new NotBlankAnnotationPlugin();
  }

  @Bean
  public PatternAnnotationPlugin patternPlugin() {
    return new PatternAnnotationPlugin();
  }
}

```